function[d] = felw2st(x,m,p,rep)
% function[d] = felw2st(x,m,p,rep)
% FELW2ST.M computes the 2-step feasible exact Whittle likelihood
%       estimator. 
%
%		Reference: Shimotsu and Phillips (2004), Shimotsu (2004)
%
%										   Katsumi Shimotsu, July 2004
%
%		Depenency: this code requires fracdiff.m and veltaper.m
%
% 		INPUT	x: data (n*1 vector)
%				m: truncation number
%               p: the order of the taper, p = 2 or 3, 3 is preferable
%				rep: number of steps
%
%____________________________________________________________________________

[n,nn] = size(x);

d = fminbnd('veltaper',-1,3,[],x,m,p);    % 1st step

for r = 1:rep

    [score,hess] = dewhitobj(d,x,m);
    d = d - score/max(hess,2);
    
end


function[dr,ddr] = dewhitobj(d,x,m)
% Computes the score and Hessian for the two-step FELW estimator
%
% Defintition of the dft and periodogram:
%
%		Ixx(k) = w(x)*conj(w(x)),
%
%	where
%								  N
%		w(k) =  (2*pi*n)^(-1/2)  sum  x(t)*exp(i*2*pi*(k-1)*t/N), 1 <= k <= N.
%                  				 t=1
%____________________________________________________________________________

[n,nn] = size(x);

weight = (d<=0.5) + (1/2)*(1 + cos(-2*pi+4*pi*d))*(d>0.5)*(d<0.75);
xbar = sum(x)/n;

myu = weight*xbar + (1-weight)*x(1);

x = x - myu;

dx = fracdiff(x,d);
t = (0:1:n-1)';
lambda = 2*pi*t/n;
wdx = (2*pi*n)^(-1/2)*conj(fft(conj(dx))).*exp(i*lambda);

v = ones(n,1);
dv = fracdiff(v,d);
wdv = (2*pi*n)^(-1/2)*conj(fft(conj(dv))).*exp(i*lambda);
wdv = wdv(2:m+1);

b = 1:1:n-1;
b = b.^(-1);
b = [0; -b'];
dlogdx = filter(b,1,dx);
ddlogdx = filter(b,1,dlogdx);
wdlogdx = (2*pi*n)^(-1/2)*conj(fft(conj(dlogdx))).*exp(i*lambda);
wddlogdx = (2*pi*n)^(-1/2)*conj(fft(conj(ddlogdx))).*exp(i*lambda);
vdlogdx = wdlogdx(2:m+1);
vddlogdx = wddlogdx(2:m+1);

lambda = lambda(2:m+1);
vx = wdx(2:m+1);
Iv = vx.*conj(vx);

g = sum(Iv)/m;

dg = 2*real(vdlogdx.*conj(vx));
dg = sum(dg)/m;

ddg = 2*real(vddlogdx.*conj(vx)) + 2*vdlogdx.*conj(vdlogdx);
ddg = sum(ddg)/m;

dr = dg/g - 2*sum(log(lambda))/m;
ddr = (ddg*g-dg*dg)/g/g;
